function navigate(tabId, url) {
    chrome.tabs.update(tabId, {url: url});
}

chrome.webRequest.onBeforeRequest.addListener(
  function(info) {
    console.log("CDN53 intercepted: " + info.url + ' with: ' + info.method);
    if (info.method != 'GET')
      // don't interfer with methods other than GET
      return {cancel: false};
    // send this to the native app:
    //   { url: "..." }
    // the app replies with either:
    //   { content: "<base64-data>", type: "<mime-type>" }
    // or in case of error:
    //   { error: "<message>" }
    chrome.runtime.sendNativeMessage('com.mrb.txtresolver',
        { url: info.url },
        function(response) {
            if ('error' in response)
                navigate(info.tabId, 'data:text/plain,' +
                    response['error'].replace(/\n/g, '%0a'));
            else
                navigate(info.tabId, 'data:' + response['type'] + ';base64,' +
                    response['content']);
        });
    // TODO: I did not find a way to send and receive a message synchronously to
    // return the correct data: URI right away, so instead a blank page is returned
    // and the asynchronous message response just calls navigate(). A downside of this
    // approach is that a page cannot include, say, a CDN53 image, because the entire
    // tab will be redirected to the CDN53 image. But I guess this is good enough for a
    // proof-of-concept...
    return {redirectUrl: 'data:text/html,'};
  },  
  // filters
  {
    urls: [
      "http://*/*" // the manifest already filters on http://*.cdn53/*
    ]
  },  
  // extraInfoSpec
  ["blocking"]
);

